@@ -1,42 +1,45 @@
 import re
-from abc import ABC, abstractmethod
-from typing import Any, Dict, List, Mapping, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, List
 
 from django.template import Context, Node, NodeList, TemplateSyntaxError
-from django.template.base import FilterExpression, Lexer, Parser, VariableNode
+from django.template.base import Parser, VariableNode
 
-Expression = Union[FilterExpression, "DynamicFilterExpression"]
-RuntimeKwargsInput = Dict[str, Union[Expression, "Operator"]]
-RuntimeKwargPairsInput = List[Tuple[str, Union[Expression, "Operator"]]]
+from django_components.util.template_parser import parse_template
 
-
-class Operator(ABC):
-    """
-    Operator describes something that somehow changes the inputs
-    to template tags (the `{% %}`).
-
-    For example, a SpreadOperator inserts one or more kwargs at the
-    specified location.
-    """
-
-    @abstractmethod
-    def resolve(self, context: Context) -> Any: ...  # noqa E704
-
-
-class SpreadOperator(Operator):
-    """Operator that inserts one or more kwargs at the specified location."""
-
-    def __init__(self, expr: Expression) -> None:
-        self.expr = expr
-
-    def resolve(self, context: Context) -> Dict[str, Any]:
-        data = self.expr.resolve(context)
-        if not isinstance(data, dict):
-            raise RuntimeError(f"Spread operator expression must resolve to a Dict, got {data}")
-        return data
+if TYPE_CHECKING:
+    from django_components.util.template_tag import TagParam
 
 
 class DynamicFilterExpression:
+    """
+    To make working with Django templates easier, we allow to use (nested) template tags `{% %}`
+    inside of strings that are passed to our template tags, e.g.:
+
+    ```django
+    {% component "my_comp" value_from_tag="{% gen_dict %}" %}
+    ```
+
+    We call this the "dynamic" or "nested" expression.
+
+    A string is marked as a dynamic expression only if it contains any one
+    of `{{ }}`, `{% %}`, or `{# #}`.
+
+    If the expression consists of a single tag, with no extra text, we return the tag's
+    value directly. E.g.:
+
+    ```django
+    {% component "my_comp" value_from_tag="{% gen_dict %}" %}
+    ```
+
+    will pass a dictionary to the component input `value_from_tag`.
+
+    But if the text already contains spaces or more tags, e.g.
+
+    `{% component "my_comp" value_from_tag=" {% gen_dict %} " %}`
+
+    Then we treat it as a regular template and pass it as string.
+    """
+
     def __init__(self, parser: Parser, expr_str: str) -> None:
         if not is_dynamic_expression(expr_str):
             raise TemplateSyntaxError(f"Not a valid dynamic expression: '{expr_str}'")
@@ -47,8 +50,7 @@ class DynamicFilterExpression:
         # Copy the Parser, and pass through the tags and filters available
         # in the current context. Thus, if user calls `{% load %}` inside
         # the expression, it won't spill outside.
-        lexer = Lexer(self.expr)
-        tokens = lexer.tokenize()
+        tokens = parse_template(self.expr)
         expr_parser = Parser(tokens=tokens)
         expr_parser.tags = {**parser.tags}
         expr_parser.filters = {**parser.filters}
@@ -79,18 +81,18 @@ class DynamicFilterExpression:
             # to avoid it being stringified
             if isinstance(node, VariableNode):
                 return node.filter_expression.resolve(context)
-            else:
-                # For any other tags `{% %}`, we're at a mercy of the authors, and
-                # we don't know if the result comes out stringified or not.
-                return node.render(context)
-        else:
-            # Lastly, if there's multiple nodes, we render it to a string
-            #
-            # NOTE: When rendering a NodeList, it expects that each node is a string.
-            # However, we want to support tags that return non-string results, so we can pass
-            # them as inputs to components. So we wrap the nodes in `StringifiedNode`
-            nodelist = NodeList(StringifiedNode(node) for node in self.nodelist)
-            return nodelist.render(context)
+
+            # For any other tags `{% %}`, we're at a mercy of the authors, and
+            # we don't know if the result comes out stringified or not.
+            return node.render(context)
+
+        # Lastly, if there's multiple nodes, we render it to a string
+        #
+        # NOTE: When rendering a NodeList, it expects that each node is a string.
+        # However, we want to support tags that return non-string results, so we can pass
+        # them as inputs to components. So we wrap the nodes in `StringifiedNode`
+        nodelist = NodeList(StringifiedNode(node) for node in self.nodelist)
+        return nodelist.render(context)
 
 
 class StringifiedNode(Node):
@@ -103,80 +105,17 @@ class StringifiedNode(Node):
         return str(result)
 
 
-class RuntimeKwargs:
-    def __init__(self, kwargs: RuntimeKwargsInput) -> None:
-        self.kwargs = kwargs
-
-    def resolve(self, context: Context) -> Dict[str, Any]:
-        resolved_kwargs = safe_resolve_dict(context, self.kwargs)
-        return process_aggregate_kwargs(resolved_kwargs)
-
-
-class RuntimeKwargPairs:
-    def __init__(self, kwarg_pairs: RuntimeKwargPairsInput) -> None:
-        self.kwarg_pairs = kwarg_pairs
-
-    def resolve(self, context: Context) -> List[Tuple[str, Any]]:
-        resolved_kwarg_pairs: List[Tuple[str, Any]] = []
-        for key, kwarg in self.kwarg_pairs:
-            if isinstance(kwarg, SpreadOperator):
-                spread_kwargs = kwarg.resolve(context)
-                for spread_key, spread_value in spread_kwargs.items():
-                    resolved_kwarg_pairs.append((spread_key, spread_value))
-            else:
-                resolved_kwarg_pairs.append((key, kwarg.resolve(context)))
-
-        return resolved_kwarg_pairs
-
-
-def is_identifier(value: Any) -> bool:
-    if not isinstance(value, str):
-        return False
-    if not value.isidentifier():
-        return False
-    return True
-
-
-def safe_resolve_list(context: Context, args: List[Expression]) -> List:
-    return [arg.resolve(context) for arg in args]
-
-
-def safe_resolve_dict(
-    context: Context,
-    kwargs: Dict[str, Union[Expression, "Operator"]],
-) -> Dict[str, Any]:
-    result = {}
-
-    for key, kwarg in kwargs.items():
-        # If we've come across a Spread Operator (...), we insert the kwargs from it here
-        if isinstance(kwarg, SpreadOperator):
-            spread_dict = kwarg.resolve(context)
-            if spread_dict is not None:
-                for spreadkey, spreadkwarg in spread_dict.items():
-                    result[spreadkey] = spreadkwarg
-        else:
-            result[key] = kwarg.resolve(context)
-    return result
-
-
-def resolve_string(
-    s: str,
-    parser: Optional[Parser] = None,
-    context: Optional[Mapping[str, Any]] = None,
-) -> str:
-    parser = parser or Parser([])
-    context = Context(context or {})
-    return parser.compile_filter(s).resolve(context)
-
-
-def is_kwarg(key: str) -> bool:
-    return "=" in key
-
-
 def is_aggregate_key(key: str) -> bool:
+    key = key.strip()
     # NOTE: If we get a key that starts with `:`, like `:class`, we do not split it.
     # This syntax is used by Vue and AlpineJS.
-    return ":" in key and not key.startswith(":")
+    return (
+        ":" in key
+        # `:` or `:class` is NOT ok
+        and not key.startswith(":")
+        # `attrs:class` is OK, but `attrs:` is NOT ok
+        and bool(key.split(":", maxsplit=1)[1])
+    )
 
 
 # A string that must start and end with quotes, and somewhere inside includes
@@ -188,51 +127,24 @@ DYNAMIC_EXPR_RE = re.compile(
         comment_tag=r"(?:\{#.*?#\})",
         start_quote=r"(?P<quote>['\"])",  # NOTE: Capture group so we check for the same quote at the end
         end_quote=r"(?P=quote)",
-    )
+    ),
 )
 
 
 def is_dynamic_expression(value: Any) -> bool:
     # NOTE: Currently dynamic expression need at least 6 characters
-    # for the opening and closing tags, and quotes
-    MIN_EXPR_LEN = 6
+    # for the opening and closing tags, and quotes, e.g. `"`, `{%`, `%}` in `" some text {% ... %}"`
+    MIN_EXPR_LEN = 6  # noqa: N806
 
     if not isinstance(value, str) or not value or len(value) < MIN_EXPR_LEN:
         return False
 
     # Is not wrapped in quotes, or does not contain any tags
-    if not DYNAMIC_EXPR_RE.match(value):
-        return False
-
-    return True
+    return bool(DYNAMIC_EXPR_RE.match(value))
 
 
-def is_spread_operator(value: Any) -> bool:
-    if not isinstance(value, str) or not value:
-        return False
-
-    return value.startswith("...")
-
-
-# A string that starts with `...1=`, `...29=`, etc.
-# We convert the spread syntax to this, so Django parses
-# it as a kwarg, so it remains in the original position.
-#
-# So from `...dict`, we make `...1=dict`
-#
-# That way it's trivial to merge the kwargs after the spread
-# operator is replaced with actual values.
-INTERNAL_SPREAD_OPERATOR_RE = re.compile(r"^\.\.\.\d+=")
-
-
-def is_internal_spread_operator(value: Any) -> bool:
-    if not isinstance(value, str) or not value:
-        return False
-
-    return bool(INTERNAL_SPREAD_OPERATOR_RE.match(value))
-
-
-def process_aggregate_kwargs(kwargs: Mapping[str, Any]) -> Dict[str, Any]:
+# TODO - Move this out into a plugin?
+def process_aggregate_kwargs(params: List["TagParam"]) -> List["TagParam"]:
     """
     This function aggregates "prefixed" kwargs into dicts. "Prefixed" kwargs
     start with some prefix delimited with `:` (e.g. `attrs:`).
@@ -285,27 +197,67 @@ def process_aggregate_kwargs(kwargs: Mapping[str, Any]) -> Dict[str, Any]:
     This provides sufficient flexiblity to make it easy for component users to provide
     "fallthrough attributes", and sufficiently easy for component authors to process
     that input while still being able to provide their own keys.
+
     """
-    processed_kwargs = {}
+    from django_components.util.template_tag import TagParam  # noqa: PLC0415
+
+    _check_kwargs_for_agg_conflict(params)
+
+    processed_params = []
+    seen_keys = set()
     nested_kwargs: Dict[str, Dict[str, Any]] = {}
-    for key, val in kwargs.items():
-        if not is_aggregate_key(key):
-            processed_kwargs[key] = val
+    for param in params:
+        # Positional args
+        if param.key is None:
+            processed_params.append(param)
             continue
 
-        # NOTE: Trim off the prefix from keys
-        prefix, sub_key = key.split(":", 1)
-        if prefix not in nested_kwargs:
-            nested_kwargs[prefix] = {}
-        nested_kwargs[prefix][sub_key] = val
+        # Regular kwargs without `:` prefix
+        if not is_aggregate_key(param.key):
+            outer_key = param.key
+            inner_key = None
+            seen_keys.add(outer_key)
+            processed_params.append(param)
+            continue
+
+        # NOTE: Trim off the outer_key from keys
+        outer_key, inner_key = param.key.split(":", 1)
+        if outer_key not in nested_kwargs:
+            nested_kwargs[outer_key] = {}
+        nested_kwargs[outer_key][inner_key] = param.value
 
     # Assign aggregated values into normal input
     for key, val in nested_kwargs.items():
-        if key in processed_kwargs:
+        if key in seen_keys:
             raise TemplateSyntaxError(
                 f"Received argument '{key}' both as a regular input ({key}=...)"
-                f" and as an aggregate dict ('{key}:key=...'). Must be only one of the two"
+                f" and as an aggregate dict ('{key}:key=...'). Must be only one of the two",
             )
-        processed_kwargs[key] = val
+        processed_params.append(TagParam(key=key, value=val))
 
-    return processed_kwargs
+    return processed_params
+
+
+def _check_kwargs_for_agg_conflict(params: List["TagParam"]) -> None:
+    seen_regular_kwargs = set()
+    seen_agg_kwargs = set()
+
+    for param in params:
+        # Ignore positional args
+        if param.key is None:
+            continue
+
+        is_agg_kwarg = is_aggregate_key(param.key)
+        if (
+            (is_agg_kwarg and (param.key in seen_regular_kwargs))
+            or (not is_agg_kwarg and (param.key in seen_agg_kwargs))
+        ):  # fmt: skip
+            raise TemplateSyntaxError(
+                f"Received argument '{param.key}' both as a regular input ({param.key}=...)"
+                f" and as an aggregate dict ('{param.key}:key=...'). Must be only one of the two",
+            )
+
+        if is_agg_kwarg:
+            seen_agg_kwargs.add(param.key)
+        else:
+            seen_regular_kwargs.add(param.key)
