@@ -5,18 +5,15 @@ from typing import TYPE_CHECKING, List, NamedTuple
 from django.template import TemplateSyntaxError
 from django.utils.module_loading import import_string
 
-from django_components.expression import resolve_string
-from django_components.template_parser import VAR_CHARS
 from django_components.util.misc import is_str_wrapped_in_quotes
 
 if TYPE_CHECKING:
     from django_components.component_registry import ComponentRegistry
 
 
-# Forward slash is added so it's possible to define components like
-# `{% MyComp %}..{% /MyComp %}`
-TAG_CHARS = VAR_CHARS + r"/"
-TAG_RE = re.compile(r"^[{chars}]+$".format(chars=TAG_CHARS))
+# Require the start / end tags to contain NO spaces and only these characters
+TAG_CHARS = r"\w\-\:\@\.\#/"
+TAG_RE = re.compile(rf"^[{TAG_CHARS}]+$")
 
 
 class TagResult(NamedTuple):
@@ -109,6 +106,7 @@ class TagFormatterABC(abc.ABC):
 
         Returns:
             str: The formatted start tag.
+
         """
         ...
 
@@ -122,6 +120,7 @@ class TagFormatterABC(abc.ABC):
 
         Returns:
             str: The formatted end tag.
+
         """
         ...
 
@@ -134,7 +133,7 @@ class TagFormatterABC(abc.ABC):
         which is a tuple of `(component_name, remaining_tokens)`.
 
         Args:
-            tokens [List(str]): List of tokens passed to the component tag.
+            tokens (List[str]): List of tokens passed to the component tag.
 
         Returns:
             TagResult: Parsed component name and remaining tokens.
@@ -163,16 +162,15 @@ class TagFormatterABC(abc.ABC):
         ```python
         TagResult('my_comp', ['key=val', 'key2=val2'])
         ```
+
         """
         ...
 
 
 class InternalTagFormatter:
-    """
-    Internal wrapper around user-provided TagFormatters, so that we validate the outputs.
-    """
+    """Internal wrapper around user-provided TagFormatters, so that we validate the outputs."""
 
-    def __init__(self, tag_formatter: TagFormatterABC):
+    def __init__(self, tag_formatter: TagFormatterABC) -> None:
         self.tag_formatter = tag_formatter
 
     def start_tag(self, name: str) -> str:
@@ -195,13 +193,13 @@ class InternalTagFormatter:
         if not tag:
             raise ValueError(
                 f"{self.tag_formatter.__class__.__name__} returned an invalid tag for {tag_type}: '{tag}'."
-                f" Tag cannot be empty"
+                f" Tag cannot be empty",
             )
 
         if not TAG_RE.match(tag):
             raise ValueError(
                 f"{self.tag_formatter.__class__.__name__} returned an invalid tag for {tag_type}: '{tag}'."
-                f" Tag must contain only following chars: {TAG_CHARS}"
+                f" Tag must contain only following chars: {TAG_CHARS}",
             )
 
 
@@ -225,42 +223,23 @@ class ComponentFormatter(TagFormatterABC):
     ```
     """
 
-    def __init__(self, tag: str):
+    def __init__(self, tag: str) -> None:
         self.tag = tag
 
-    def start_tag(self, name: str) -> str:
+    def start_tag(self, _name: str) -> str:
         return self.tag
 
-    def end_tag(self, name: str) -> str:
+    def end_tag(self, _name: str) -> str:
         return f"end{self.tag}"
 
     def parse(self, tokens: List[str]) -> TagResult:
-        tag, *args = tokens
+        _tag, *args = tokens
 
         if not args:
             raise TemplateSyntaxError(f"{self.__class__.__name__}: Component tag did not receive tag name")
 
-        # If the first arg is a kwarg, not a positional arg, then look for the "name" kwarg
-        # for component name.
-        if "=" in args[0]:
-            comp_name = None
-            final_args = []
-            for kwarg in args:
-                if not kwarg.startswith("name="):
-                    final_args.append(kwarg)
-                    continue
-
-                if comp_name:
-                    raise TemplateSyntaxError(
-                        f"ComponentFormatter: 'name' kwarg for component '{comp_name}'" " was defined more than once."
-                    )
-
-                # NOTE: We intentionally do NOT add to `final_args` here
-                # because we want to remove the the `name=` kwarg from args list
-                comp_name = kwarg[5:]
-        else:
-            comp_name = args.pop(0)
-            final_args = args
+        # If the first arg is a kwarg, then clearly the component name is not set.
+        comp_name = None if "=" in args[0] else args.pop(0)
 
         if not comp_name:
             raise TemplateSyntaxError("Component name must be a non-empty quoted string, e.g. 'my_comp'")
@@ -269,9 +248,9 @@ class ComponentFormatter(TagFormatterABC):
             raise TemplateSyntaxError(f"Component name must be a string 'literal', got: {comp_name}")
 
         # Remove the quotes
-        comp_name = resolve_string(comp_name)
+        comp_name = comp_name[1:-1]
 
-        return TagResult(comp_name, final_args)
+        return TagResult(comp_name, args)
 
 
 class ShorthandComponentFormatter(TagFormatterABC):
