@@ -1,6 +1,6 @@
 """Helper types for IDEs."""
 
-from django_components.util.types import Annotated
+from typing_extensions import Annotated
 
 css = Annotated[str, "css"]
 django_html = Annotated[str, "django_html"]
