@@ -13,17 +13,16 @@ def generate(alphabet: str, size: int) -> str:
     mask = 1
     if alphabet_len > 1:
         mask = (2 << int(log(alphabet_len - 1) / log(2))) - 1
-    step = int(ceil(1.6 * mask * size / alphabet_len))
+    step = int(ceil(1.6 * mask * size / alphabet_len))  # noqa: RUF046
 
-    id = ""
+    id_str = ""
     while True:
         random_bytes = bytearray(urandom(step))
 
         for i in range(step):
             random_byte = random_bytes[i] & mask
-            if random_byte < alphabet_len:
-                if alphabet[random_byte]:
-                    id += alphabet[random_byte]
+            if random_byte < alphabet_len and alphabet[random_byte]:
+                id_str += alphabet[random_byte]
 
-                    if len(id) == size:
-                        return id
+                if len(id_str) == size:
+                    return id_str
